#!/bin/bash

# Example of running this script
# (configurable levels, 1 instance, tiny config, 100 frames, component 1):
#sudo ./run_hog.sh 4 1 graph_config_tiny.txt 100 1

# Sched options: 0:e2e, 1:coarse, 2:fine, 3:unrolled-coarse, 4:configurable-levels
sched=$1

# How many copies to run at once?
instance=$2

# Which graph config file?
config=$3

# How many frames? (the video is 100 frames long, and repeats)
count=$4

# Which component should these instances be scheduled in?
component=$5

# What is the graph RTB?  This is used to determine the
# intra-task parallelism, as bound/hard-coded-period
bound=80

# Choose the executable based on access-merging config (this should not be required, but it is); the code has to be recompiled with different settings for FZ lengths
if [[ "$config" == *"major"* ]];
then
    exec="~/opencv/build/bin/example_rt_gpu_hog_major"
elif [[ "$config" == *"moderate"* ]];
then
    exec="~/opencv/build/bin/example_rt_gpu_hog_moderate"
elif [[ "$config" == *"minor"* ]];
then
    exec="~/opencv/build/bin/example_rt_gpu_hog_minor"
else
    exec="~/opencv/build/bin/example_rt_gpu_hog_none"
fi
exec="~/opencv/build/bin/example_rt_gpu_hog_timing"

experiment_dir="~/timewall/experiments/diss"
prefix="sched="$sched"_count="$count"_inst="$instance
outfile="${experiment_dir}/${prefix}"
for i in `seq 1 1 $instance`; do
        $exec \
        --make_gray true \
        --video ~/opencv/samples/data/vtest.avi \
        --display false \
        --cluster $component \
        --sched $sched --id $i \
        --count $count --graph_bound $bound \
        --num_inst 1 \
        --level_config_file ~/timewall/scripts/diss/$config > \
        $outfile"_"$i.log &
    echo "done with $i, sleep to make sure PGM^RT worked well constructing graphs"
    sleep 25
done

echo "Finished initializing graphs"
wait $!

ss_fn="${experiment_dir}/source-sink-info_"$prefix

echo -n "" > $ss_fn
for i in `seq 1 1 $instance`; do
    cat "${experiment_dir}/${prefix}_${i}.log" | grep "source\|sink" >> $ss_fn
done
