#!/usr/bin/env python
import subprocess as sp
import time
import signal
import random
import os

LIBLITMUS = "~/liblitmus"
FEATHER_TRACE = "~/feather-trace-tools"
EXP_DIR = "~/timewall/experiments/diss"

RUN_COMPONENT_B = True
RUN_COMPONENT_C = True

def main():
	taskset = []

	def cleanup_rtspin(*args):
		for task in taskset:
			task.kill()
		exit()

	signal.signal(signal.SIGINT, cleanup_rtspin)
	signal.signal(signal.SIGSEGV, cleanup_rtspin)
	signal.signal(signal.SIGTERM, cleanup_rtspin)

	timeslices = [5, 10, 20, 40, 80, 160]
	timeslices = [20]

	#for loop in range(1):
	for loop in range(len(timeslices)):
		random.seed(314159265358)

		trace = sp.Popen([LIBLITMUS + '/setsched EXT-RES'], shell=True)
		trace.wait()
		time.sleep(2.0)

		trace = sp.Popen([FEATHER_TRACE + '/st-trace-schedule', 'temp-trace'], stdin=sp.PIPE)
		print('starting up tracing framework...')
		time.sleep(5.0)

		########################################################
		# Component Setup
		########################################################

		print('setting up components')
		MAJOR_CYCLE = '320'
		timeslice = timeslices[loop]
		subdiv = int(int(MAJOR_CYCLE) / (2* timeslice))
		comp1_intervals = []
		comp2_intervals = []
		for i in range(subdiv):
			comp1_intervals.append('[' + str((i*2+0)*timeslice) + ',' + str((i*2+1)*timeslice) + ')')
			comp2_intervals.append('[' + str((i*2+1)*timeslice) + ',' + str((i*2+2)*timeslice) + ')')
		comp3_intervals = ['[0,' + str(int(MAJOR_CYCLE)-1) + ']']

		# Component 1: cores 1-4, Hog
		for i in range(1,5):
			params = [LIBLITMUS + '/resctl', '-c', str(i), '-n', '1', '-t',
					  'table-driven', '-m', MAJOR_CYCLE]
			params.extend(comp1_intervals)
			print(params)
			component = sp.Popen(params)

		# Component 2: cores 1-4, Lane Detection
		for i in range(1,5):
			params = [LIBLITMUS + '/resctl', '-c', str(i), '-n', '2', '-t',
					  'table-driven', '-m', MAJOR_CYCLE]
			params.extend(comp2_intervals)
			print(params)
			component = sp.Popen(params)

		# Component 3: cores 5-7, Planning
		for i in range(5,8):
			params = [LIBLITMUS + '/resctl', '-c', str(i), '-n', '3', '-t',
					  'table-driven', '-m', MAJOR_CYCLE]
			params.extend(comp3_intervals)
			print(params)
			component = sp.Popen(params)

		# https://www.kernel.org/doc/Documentation/x86/intel_rdt_ui.txt
		cache_ctrl = sp.Popen([r'echo "L3:0=f;1=f\nMB:0=40;1=40" > /sys/fs/resctrl/c0/schemata'],
								shell=True)
		cache_ctrl.wait()
		cache_ctrl = sp.Popen([r'echo "L3:0=70;1=70\nMB:0=30;1=30" > /sys/fs/resctrl/c1/schemata'],
								shell=True)
		cache_ctrl.wait();
		cache_ctrl = sp.Popen([r'echo "L3:0=380;1=380\nMB:0=30;1=30" > /sys/fs/resctrl/c2/schemata'],
								shell=True)
		cache_ctrl.wait()

		########################################################
		# Component Setup
		########################################################

		taskset = []
		
		hog_iter = 2
		num_frames = 25000
		access_merge = "major"
		config = "graph_config_single_" + access_merge + ".txt"
		print('starting up HOG')
		# Hog params: <unused> <number of instances> <config file> <number of frames> <reservation ID>
		hog_task = sp.Popen(['./run_hog.sh', '4', str(hog_iter), config, str(num_frames), '1'])
		print(hog_task.pid)
		cache_ctrl = sp.Popen(['echo ' + str(hog_task.pid) + 
								r' >> /sys/fs/resctrl/c0/tasks'], shell=True)
		cache_ctrl.wait()
		time.sleep(25 * hog_iter)
		print('finished HOG startup')


		########################################################
		# Lane Detection
		########################################################

		lane_detection_log_file = None
		if RUN_COMPONENT_B:
			print("\nStarting lane detection")
			num_lane_detection_frames = 2000
			lane_detection_log_file = open(EXP_DIR + "/lane_detection.log", 'w')
			lane_detection_exe = "~/opencv/build/bin/example_rt_gpu_lane_detection"

			lane_detection = sp.Popen([lane_detection_exe,
                                                   "--folder",
                                                   "~/carla_images/scenario_5/",
                                                   "--display", "false",
                                                   "--count", str(num_lane_detection_frames),
                                                   "--rt_period", "200",
                                                   "--rt_deadline", "200"],
                                                  stdout=lane_detection_log_file)
                                                   
			cache_ctrl = sp.Popen(['echo ' + str(lane_detection.pid) + \
                                               r' >> /sys/fs/resctrl/c1/tasks'], shell=True)
			cache_ctrl.wait()
			taskset.append(lane_detection)

			time.sleep(30)


		########################################################
		# Motion Planning
		########################################################

		rrt_log_file = None
		pure_pursuit_log_file = None
		if RUN_COMPONENT_C:
			print("\nStarting RRT")
			rrt_log_file = open(EXP_DIR + "/rrt.log", 'w')
			rrt = sp.Popen(["python3",
                                        "~/motion_planning_rt/python_src/rrts/main_rrt2D.py"],
                                       stdout=rrt_log_file)
			cache_ctrl = sp.Popen(['echo ' + str(rrt.pid) + \
                                               r' >> /sys/fs/resctrl/c2/tasks'], shell=True)
			cache_ctrl.wait()
			taskset.append(rrt)
			
			time.sleep(2)
			
			print("\nStarting Pure Pursuit")
			pure_pursuit_log_file = open(EXP_DIR + "/pure_pursuit.log", 'w')
			pure_pursuit = sp.Popen(["python3",
                                                 "~/pure_pursuit_rt/pure_pursuit.py"],
                                                stdout=pure_pursuit_log_file)
			cache_ctrl = sp.Popen(['echo ' + str(pure_pursuit.pid) + \
                                               r' >> /sys/fs/resctrl/c2/tasks'], shell=True)
			cache_ctrl.wait()
			taskset.append(pure_pursuit)
                        
			time.sleep(2)
                        
                        
		########################################################
		# Release tasks, then kill them all after some time
		########################################################
		
		releaser = sp.Popen([LIBLITMUS + '/release_ts', '-q', MAJOR_CYCLE])
		releaser.wait()
			
		# Wait for Hog to finish
		hog_task.wait()

		# Kill competing work
		print("Hog has completed. Killing competing work...")
		for task in taskset:
			task.kill()

		print("Waiting for competing work to terminate...")
		for task in taskset:
			task.wait()

		if lane_detection_log_file != None:
			lane_detection_log_file.close()
		if rrt_log_file != None:
			rrt_log_file.close()
		if pure_pursuit_log_file != None:
			pure_pursuit_log_file.close()

		print('Done. Terminating st-trace...')
		trace.communicate(b'\n')
		trace.wait()

		# Set the scheduler back to Linux
		trace = sp.Popen([LIBLITMUS + '/setsched', 'Linux'], shell=False)
		trace.wait()

		cache_ctrl = sp.Popen(['> /sys/fs/resctrl/c0/tasks'], shell=True)
		cache_ctrl.wait()
		cache_ctrl = sp.Popen(['> /sys/fs/resctrl/c1/tasks'], shell=True)
		cache_ctrl.wait()
		cache_ctrl = sp.Popen(['> /sys/fs/resctrl/c2/tasks'], shell=True)
		cache_ctrl.wait()

		tmp = sp.Popen([FEATHER_TRACE + r'/st-job-stats ' +
				r'schedule_host\=*_scheduler\=EXT-RES_trace\=temp-trace_cpu\=*.bin ' +
				'> temp' + str(timeslice) + '.log'], shell=True)
		tmp.wait()

		# Parse out the response times
		parse_resp = sp.Popen('python parse_jobresponse.py ./ temp' + str(timeslice) + \
                                      '.log temp' + str(timeslice) + '_resp.log', shell=True)
		parse_resp.wait()

		# Move all of the experiment files to a directory (change based on which
		# components are active)
		active_comps = "ABC" if RUN_COMPONENT_B and RUN_COMPONENT_C else \
                               ("AB" if not RUN_COMPONENT_C else "A")
		local_exp_dir = EXP_DIR + "/hog_response/" + access_merge + "/" +\
                                str(timeslice) + "/" + active_comps + "/"
		mv_files = sp.Popen('mkdir -p ' + local_exp_dir, shell=True)
		mv_files.wait()
		mv_files = sp.Popen('mv temp' + str(timeslice) + '*.log ' + local_exp_dir,
		                    shell=True)
		mv_files.wait()
		mv_files = sp.Popen('mv ' + EXP_DIR + '/sched=4_count=' + str(num_frames) + \
                                    "_inst=" + \
                                    str(hog_iter) + "_*.log " + local_exp_dir, shell=True)
		mv_files.wait()

if __name__ == '__main__':
	main()
