###########
# Setup file for RTSS '21 partitions.
###########

import subprocess as sp
import time
import signal
import random

LIBLITMUS = "~/liblitmus"

def main():
    taskset = []

    def cleanup_rtspin(*args):
        for task in taskset:
            task.kill()
        exit()

    signal.signal(signal.SIGINT, cleanup_rtspin)
    signal.signal(signal.SIGSEGV, cleanup_rtspin)
    signal.signal(signal.SIGTERM, cleanup_rtspin)

    trace = sp.Popen([LIBLITMUS + '/setsched EXT-RES'], shell=True)
    trace.wait()
    time.sleep(5.0)

    print('setting up components')

    MAJOR_CYCLE = '200'
    comp1_intervals = [ '[0,20)', '[40,60)', '[80,100)', '[120,140)', '[160,180)' ]
    comp2_intervals = [ '[20,40)', '[60,80)', '[100,120)', '[140,160)', '[180,200)' ]

    # Component 1: cores 1-4
    for i in range(1,5):
        params = [LIBLITMUS + '/resctl',
                  '-c', str(i), '-n', '1',
                  '-t', 'table-driven', '-m', MAJOR_CYCLE]
        params.extend(comp1_intervals)
        print(params)
        component = sp.Popen(params)

    # Component 2: cores 1-4
    for i in range(1,5):
        params = [LIBLITMUS + '/resctl',
                  '-c', str(i), '-n', '2',
                  '-t', 'table-driven', '-m', MAJOR_CYCLE]
        params.extend(comp2_intervals)
        component = sp.Popen(params)

    # Component 3: cores 5-7
    for i in range(5,8):
        params = [LIBLITMUS + '/resctl',
                  '-c', str(i), '-n', '3',
                  '-t', 'table-driven', '-m', MAJOR_CYCLE]
        params.extend([ '[0,200)' ])
        component = sp.Popen(params)

    # Component 4: cores 5-7
    for i in range(5,8):
        params = [LIBLITMUS + '/resctl',
                  '-c', str(i), '-n', '4',
                  '-t', 'table-driven', '-m', MAJOR_CYCLE]
        params.extend([ '[0,200)' ])
        component = sp.Popen(params)

if __name__ == '__main__':
    main()
